@@ -42,7 +42,6 @@ import (
 	"github.com/dolthub/dolt/go/libraries/utils/config"
 	"github.com/dolthub/dolt/go/libraries/utils/tracing"
 	"github.com/dolthub/dolt/go/store/hash"
-	"github.com/dolthub/dolt/go/store/types"
 )
 
 const (
@@ -77,10 +76,6 @@ func (cmd FilterBranchCmd) Description() string {
 	return fmt.Sprintf("%s.", filterBranchDocs.ShortDesc)
 }
 
-func (cmd FilterBranchCmd) GatedForNBF(nbf *types.NomsBinFormat) bool {
-	return types.IsFormat_DOLT_1(nbf)
-}
-
 func (cmd FilterBranchCmd) Docs() *cli.CommandDocumentation {
 	ap := cmd.ArgParser()
 	return cli.NewCommandDocumentation(filterBranchDocs, ap)
