@@ -35,7 +35,6 @@ import (
 	"github.com/dolthub/dolt/go/libraries/utils/argparser"
 	"github.com/dolthub/dolt/go/store/datas"
 	"github.com/dolthub/dolt/go/store/datas/pull"
-	"github.com/dolthub/dolt/go/store/types"
 )
 
 var pushDocs = cli.CommandDocumentationContent{
@@ -66,10 +65,6 @@ func (cmd PushCmd) Description() string {
 	return "Push to a dolt remote."
 }
 
-func (cmd PushCmd) GatedForNBF(nbf *types.NomsBinFormat) bool {
-	return types.IsFormat_DOLT_1(nbf)
-}
-
 func (cmd PushCmd) Docs() *cli.CommandDocumentation {
 	ap := cmd.ArgParser()
 	return cli.NewCommandDocumentation(pushDocs, ap)
