@@ -15,6 +15,7 @@
 package merge
 
 import (
+	"bytes"
 	"context"
 	"encoding/json"
 	"errors"
@@ -162,7 +163,11 @@ func mergeProllyIndexSets(
 			if err != nil {
 				return prolly.Map{}, false, err
 			}
-			return durable.ProllyMapFromIndex(idx), true, nil
+			m := durable.ProllyMapFromIndex(idx)
+			if schema.IsKeyless(sch) {
+				m = prolly.ConvertToSecondaryKeylessIndex(m)
+			}
+			return m, true, nil
 		}
 		return prolly.Map{}, false, nil
 	}
@@ -198,6 +203,11 @@ func mergeProllyIndexSets(
 
 			var collision = false
 			merged, err := prolly.MergeMaps(ctx, rootI, mergeI, ancI, func(left, right tree.Diff) (tree.Diff, bool) {
+				if left.Type == right.Type && bytes.Equal(left.To, right.To) {
+					// convergent edit
+					return left, true
+				}
+
 				collision = true
 				return tree.Diff{}, true
 			})
@@ -347,21 +357,24 @@ OUTER:
 
 // getMutableSecondaryIdxs returns a MutableSecondaryIdx for each secondary index
 // defined in |schema| and |tbl|.
-func getMutableSecondaryIdxs(ctx context.Context, schema schema.Schema, tbl *doltdb.Table) ([]MutableSecondaryIdx, error) {
+func getMutableSecondaryIdxs(ctx context.Context, sch schema.Schema, tbl *doltdb.Table) ([]MutableSecondaryIdx, error) {
 	indexSet, err := tbl.GetIndexSet(ctx)
 	if err != nil {
 		return nil, err
 	}
 
-	mods := make([]MutableSecondaryIdx, schema.Indexes().Count())
-	for i, index := range schema.Indexes().AllIndexes() {
-		idx, err := indexSet.GetIndex(ctx, schema, index.Name())
+	mods := make([]MutableSecondaryIdx, sch.Indexes().Count())
+	for i, index := range sch.Indexes().AllIndexes() {
+		idx, err := indexSet.GetIndex(ctx, sch, index.Name())
 		if err != nil {
 			return nil, err
 		}
 		m := durable.ProllyMapFromIndex(idx)
+		if schema.IsKeyless(sch) {
+			m = prolly.ConvertToSecondaryKeylessIndex(m)
+		}
 
-		mods[i] = NewMutableSecondaryIdx(m, schema, index, m.Pool())
+		mods[i] = NewMutableSecondaryIdx(m, sch, index, m.Pool())
 	}
 
 	return mods, nil
