@@ -24,19 +24,18 @@ import (
 
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/dolthub/go-mysql-server/sql/mysql_db"
-	"github.com/dolthub/vitess/go/vt/proto/query"
 	"gopkg.in/src-d/go-errors.v1"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env/actions/commitwalk"
+	"github.com/dolthub/dolt/go/libraries/doltcore/ref"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dprocedures"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dtables"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/globalstate"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/writer"
 	"github.com/dolthub/dolt/go/libraries/doltcore/table/editor"
 )
 
@@ -279,122 +278,28 @@ func (db Database) GetTableInsensitive(ctx *sql.Context, tblName string) (sql.Ta
 	}
 
 	root, err := db.GetRoot(ctx)
-
 	if err != nil {
 		return nil, false, err
 	}
 
-	return db.GetTableInsensitiveWithRoot(ctx, root, tblName)
-}
-
-func (db Database) GetTableInsensitiveWithRoot(ctx *sql.Context, root *doltdb.RootValue, tblName string) (sql.Table, bool, error) {
-	lwrName := strings.ToLower(tblName)
-
-	sess := dsess.DSessFromSess(ctx.Session)
-
-	// NOTE: system tables are not suitable for caching
-	// TODO: these tables that cache a root value at construction time should not, they need to get it from the session
-	//  at runtime
-	switch {
-	case strings.HasPrefix(lwrName, doltdb.DoltDiffTablePrefix):
-		suffix := tblName[len(doltdb.DoltDiffTablePrefix):]
-		head, err := sess.GetHeadCommit(ctx, db.name)
-		if err != nil {
-			return nil, false, err
-		}
-		dt, err := dtables.NewDiffTable(ctx, suffix, db.ddb, root, head)
-		if err != nil {
-			return nil, false, err
-		}
-		return dt, true, nil
-	case strings.HasPrefix(lwrName, doltdb.DoltCommitDiffTablePrefix):
-		suffix := tblName[len(doltdb.DoltCommitDiffTablePrefix):]
-		dt, err := dtables.NewCommitDiffTable(ctx, suffix, db.ddb, root)
-		if err != nil {
-			return nil, false, err
-		}
-		return dt, true, nil
-	case strings.HasPrefix(lwrName, doltdb.DoltHistoryTablePrefix):
-		baseTableName := tblName[len(doltdb.DoltHistoryTablePrefix):]
-		baseTable, ok, err := db.GetTableInsensitiveWithRoot(ctx, root, baseTableName)
-		if err != nil {
-			return nil, false, err
-		}
-		if !ok {
-			return nil, false, nil
-		}
-
-		head, err := sess.GetHeadCommit(ctx, db.name)
-		if err != nil {
-			return nil, false, err
-		}
-
-		return NewHistoryTable(baseTable.(*AlterableDoltTable).DoltTable, db.ddb, head), true, nil
-	case strings.HasPrefix(lwrName, doltdb.DoltConfTablePrefix):
-		suffix := tblName[len(doltdb.DoltConfTablePrefix):]
-		dt, err := dtables.NewConflictsTable(ctx, suffix, root, dtables.RootSetter(db))
-		if err != nil {
-			return nil, false, err
-		}
-		return dt, true, nil
-	case strings.HasPrefix(lwrName, doltdb.DoltConstViolTablePrefix):
-		suffix := tblName[len(doltdb.DoltConstViolTablePrefix):]
-		dt, err := dtables.NewConstraintViolationsTable(ctx, suffix, root, dtables.RootSetter(db))
-		if err != nil {
-			return nil, false, err
-		}
-		return dt, true, nil
+	head, err := ds.GetHeadCommit(ctx, db.Name())
+	if err != nil {
+		return nil, false, err
 	}
 
-	// NOTE: system tables are not suitable for caching
-	var dt sql.Table
-	found := false
-	switch lwrName {
-	case doltdb.LogTableName:
-		head, err := sess.GetHeadCommit(ctx, db.name)
-		if err != nil {
-			return nil, false, err
-		}
-		dt, found = dtables.NewLogTable(ctx, db.ddb, head), true
-	case doltdb.DiffTableName:
-		head, err := sess.GetHeadCommit(ctx, db.name)
-		if err != nil {
-			return nil, false, err
-		}
-		dt, found = dtables.NewUnscopedDiffTable(ctx, db.ddb, head), true
-	case doltdb.TableOfTablesInConflictName:
-		dt, found = dtables.NewTableOfTablesInConflict(ctx, db.name, db.ddb), true
-	case doltdb.TableOfTablesWithViolationsName:
-		dt, found = dtables.NewTableOfTablesConstraintViolations(ctx, root), true
-	case doltdb.BranchesTableName:
-		dt, found = dtables.NewBranchesTable(ctx, db.ddb), true
-	case doltdb.RemotesTableName:
-		dt, found = dtables.NewRemotesTable(ctx, db.ddb), true
-	case doltdb.CommitsTableName:
-		dt, found = dtables.NewCommitsTable(ctx, db.ddb), true
-	case doltdb.CommitAncestorsTableName:
-		dt, found = dtables.NewCommitAncestorsTable(ctx, db.ddb), true
-	case doltdb.StatusTableName:
-		dt, found = dtables.NewStatusTable(ctx, db.name, db.ddb, dsess.NewSessionStateAdapter(sess.Session, db.name, map[string]env.Remote{}, map[string]env.BranchConfig{}, map[string]env.Remote{}), db.drw), true
-	}
-	if found {
-		return dt, found, nil
-	}
-
-	return db.getTable(ctx, root, tblName)
+	return db.getTableInsensitive(ctx, head, root, tblName)
 }
 
 // GetTableInsensitiveAsOf implements sql.VersionedDatabase
 func (db Database) GetTableInsensitiveAsOf(ctx *sql.Context, tableName string, asOf interface{}) (sql.Table, bool, error) {
-	root, err := db.rootAsOf(ctx, asOf)
-
+	head, root, err := resolveAsOf(ctx, db, asOf)
 	if err != nil {
 		return nil, false, err
 	} else if root == nil {
 		return nil, false, nil
 	}
 
-	table, ok, err := db.getTable(ctx, root, tableName)
+	table, ok, err := db.getTableInsensitive(ctx, head, root, tableName)
 	if err != nil {
 		return nil, false, err
 	}
@@ -402,6 +307,12 @@ func (db Database) GetTableInsensitiveAsOf(ctx *sql.Context, tableName string, a
 		return nil, false, nil
 	}
 
+	if doltdb.IsReadOnlySystemTable(tableName) {
+		// currently, system tables do not need to be "locked to root"
+		//  see comment below in getTableInsensitive
+		return table, ok, nil
+	}
+
 	switch table := table.(type) {
 	case *DoltTable:
 		tbl, err := table.LockedToRoot(ctx, root)
@@ -426,38 +337,123 @@ func (db Database) GetTableInsensitiveAsOf(ctx *sql.Context, tableName string, a
 	}
 }
 
-// rootAsOf returns the root of the DB as of the expression given, which may be nil in the case that it refers to an
-// expression before the first commit.
-func (db Database) rootAsOf(ctx *sql.Context, asOf interface{}) (*doltdb.RootValue, error) {
+func (db Database) getTableInsensitive(ctx *sql.Context, head *doltdb.Commit, root *doltdb.RootValue, tblName string) (sql.Table, bool, error) {
+	lwrName := strings.ToLower(tblName)
+
+	// TODO: these tables that cache a root value at construction time should not, they need to get it from the session
+	//  at runtime
+	switch {
+	case strings.HasPrefix(lwrName, doltdb.DoltDiffTablePrefix):
+		suffix := tblName[len(doltdb.DoltDiffTablePrefix):]
+		dt, err := dtables.NewDiffTable(ctx, suffix, db.ddb, root, head)
+		if err != nil {
+			return nil, false, err
+		}
+		return dt, true, nil
+
+	case strings.HasPrefix(lwrName, doltdb.DoltCommitDiffTablePrefix):
+		suffix := tblName[len(doltdb.DoltCommitDiffTablePrefix):]
+		dt, err := dtables.NewCommitDiffTable(ctx, suffix, db.ddb, root)
+		if err != nil {
+			return nil, false, err
+		}
+		return dt, true, nil
+
+	case strings.HasPrefix(lwrName, doltdb.DoltHistoryTablePrefix):
+		baseTableName := tblName[len(doltdb.DoltHistoryTablePrefix):]
+		baseTable, ok, err := db.getTable(ctx, root, baseTableName)
+		if err != nil {
+			return nil, false, err
+		}
+		if !ok {
+			return nil, false, nil
+		}
+		return NewHistoryTable(baseTable.(*AlterableDoltTable).DoltTable, db.ddb, head), true, nil
+
+	case strings.HasPrefix(lwrName, doltdb.DoltConfTablePrefix):
+		suffix := tblName[len(doltdb.DoltConfTablePrefix):]
+		dt, err := dtables.NewConflictsTable(ctx, suffix, root, dtables.RootSetter(db))
+		if err != nil {
+			return nil, false, err
+		}
+		return dt, true, nil
+
+	case strings.HasPrefix(lwrName, doltdb.DoltConstViolTablePrefix):
+		suffix := tblName[len(doltdb.DoltConstViolTablePrefix):]
+		dt, err := dtables.NewConstraintViolationsTable(ctx, suffix, root, dtables.RootSetter(db))
+		if err != nil {
+			return nil, false, err
+		}
+		return dt, true, nil
+	}
+
+	var dt sql.Table
+	found := false
+	switch lwrName {
+	case doltdb.LogTableName:
+		dt, found = dtables.NewLogTable(ctx, db.ddb, head), true
+	case doltdb.DiffTableName:
+		dt, found = dtables.NewUnscopedDiffTable(ctx, db.ddb, head), true
+	case doltdb.TableOfTablesInConflictName:
+		dt, found = dtables.NewTableOfTablesInConflict(ctx, db.name, db.ddb), true
+	case doltdb.TableOfTablesWithViolationsName:
+		dt, found = dtables.NewTableOfTablesConstraintViolations(ctx, root), true
+	case doltdb.BranchesTableName:
+		dt, found = dtables.NewBranchesTable(ctx, db.ddb), true
+	case doltdb.RemotesTableName:
+		dt, found = dtables.NewRemotesTable(ctx, db.ddb), true
+	case doltdb.CommitsTableName:
+		dt, found = dtables.NewCommitsTable(ctx, db.ddb), true
+	case doltdb.CommitAncestorsTableName:
+		dt, found = dtables.NewCommitAncestorsTable(ctx, db.ddb), true
+	case doltdb.StatusTableName:
+		sess := dsess.DSessFromSess(ctx.Session)
+		adapter := dsess.NewSessionStateAdapter(
+			sess.Session, db.name,
+			map[string]env.Remote{},
+			map[string]env.BranchConfig{},
+			map[string]env.Remote{})
+		dt, found = dtables.NewStatusTable(ctx, db.name, db.ddb, adapter, db.drw), true
+	}
+	if found {
+		return dt, found, nil
+	}
+
+	return db.getTable(ctx, root, tblName)
+}
+
+// resolveAsOf resolved given expression to a commit, if one exists.
+func resolveAsOf(ctx *sql.Context, db Database, asOf interface{}) (*doltdb.Commit, *doltdb.RootValue, error) {
+	head := db.rsr.CWBHeadRef()
 	switch x := asOf.(type) {
-	case string:
-		return db.getRootForCommitRef(ctx, x)
 	case time.Time:
-		return db.getRootForTime(ctx, x)
+		return resolveAsOfTime(ctx, db.ddb, head, x)
+	case string:
+		return resolveAsOfCommitRef(ctx, db.ddb, head, x)
 	default:
 		panic(fmt.Sprintf("unsupported AS OF type %T", asOf))
 	}
 }
 
-func (db Database) getRootForTime(ctx *sql.Context, asOf time.Time) (*doltdb.RootValue, error) {
+func resolveAsOfTime(ctx *sql.Context, ddb *doltdb.DoltDB, head ref.DoltRef, asOf time.Time) (*doltdb.Commit, *doltdb.RootValue, error) {
 	cs, err := doltdb.NewCommitSpec("HEAD")
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
-	cm, err := db.ddb.Resolve(ctx, cs, db.rsr.CWBHeadRef())
+	cm, err := ddb.Resolve(ctx, cs, head)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	hash, err := cm.HashOf()
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
-	cmItr, err := commitwalk.GetTopologicalOrderIterator(ctx, db.ddb, hash)
+	cmItr, err := commitwalk.GetTopologicalOrderIterator(ctx, ddb, hash)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	for {
@@ -465,44 +461,61 @@ func (db Database) getRootForTime(ctx *sql.Context, asOf time.Time) (*doltdb.Roo
 		if err == io.EOF {
 			break
 		} else if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 
 		meta, err := curr.GetCommitMeta(ctx)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 
 		if meta.Time().Equal(asOf) || meta.Time().Before(asOf) {
-			return curr.GetRootValue(ctx)
+			root, err := curr.GetRootValue(ctx)
+			if err != nil {
+				return nil, nil, err
+			}
+			return curr, root, nil
 		}
 	}
-
-	return nil, nil
+	return nil, nil, nil
 }
 
-func (db Database) getRootForCommitRef(ctx *sql.Context, commitRef string) (*doltdb.RootValue, error) {
-	cs, err := doltdb.NewCommitSpec(commitRef)
-	if err != nil {
-		return nil, err
+func resolveAsOfCommitRef(ctx *sql.Context, ddb *doltdb.DoltDB, head ref.DoltRef, commitRef string) (*doltdb.Commit, *doltdb.RootValue, error) {
+	if commitRef == doltdb.Working || commitRef == doltdb.Staged {
+		sess := dsess.DSessFromSess(ctx.Session)
+		root, _, err := sess.ResolveRootForRef(ctx, ctx.GetCurrentDatabase(), commitRef)
+		if err != nil {
+			return nil, nil, err
+		}
+
+		cm, err := ddb.ResolveCommitRef(ctx, head)
+		if err != nil {
+			return nil, nil, err
+		}
+		return cm, root, nil
 	}
 
-	cm, err := db.ddb.Resolve(ctx, cs, db.rsr.CWBHeadRef())
+	cs, err := doltdb.NewCommitSpec(commitRef)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
+	}
+
+	cm, err := ddb.Resolve(ctx, cs, head)
+	if err != nil {
+		return nil, nil, err
 	}
 
 	root, err := cm.GetRootValue(ctx)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
-	return root, nil
+	return cm, root, nil
 }
 
 // GetTableNamesAsOf implements sql.VersionedDatabase
 func (db Database) GetTableNamesAsOf(ctx *sql.Context, time interface{}) ([]string, error) {
-	root, err := db.rootAsOf(ctx, time)
+	_, root, err := resolveAsOf(ctx, db, time)
 	if err != nil {
 		return nil, err
 	} else if root == nil {
@@ -597,8 +610,6 @@ func filterDoltInternalTables(tblNames []string) []string {
 	return result
 }
 
-var hashType = sql.MustCreateString(query.Type_TEXT, 32, sql.Collation_ascii_bin)
-
 // GetRoot returns the root value for this database session
 func (db Database) GetRoot(ctx *sql.Context) (*doltdb.RootValue, error) {
 	sess := dsess.DSessFromSess(ctx.Session)
@@ -674,19 +685,7 @@ func (db Database) DropTable(ctx *sql.Context, tableName string) error {
 		return err
 	}
 
-	err = db.dropTableFromAiTracker(ctx, tableName)
-	if err != nil {
-		return err
-	}
-
-	return db.SetRoot(ctx, newRoot)
-}
-
-// dropTableFromAiTracker grabs the auto increment tracker and removes the table named tableName from it.
-func (db Database) dropTableFromAiTracker(ctx *sql.Context, tableName string) error {
-	sess := dsess.DSessFromSess(ctx.Session)
-	ws, err := sess.WorkingSet(ctx, db.Name())
-
+	ws, err := ds.WorkingSet(ctx, db.Name())
 	if err != nil {
 		return err
 	}
@@ -697,7 +696,7 @@ func (db Database) dropTableFromAiTracker(ctx *sql.Context, tableName string) er
 	}
 	ait.DropTable(tableName)
 
-	return nil
+	return db.SetRoot(ctx, newRoot)
 }
 
 // CreateTable creates a table with the name and schema given.
@@ -1008,7 +1007,7 @@ func (db Database) GetExternalStoredProcedures(ctx *sql.Context) ([]sql.External
 	return dprocedures.DoltProcedures, nil
 }
 
-func (db Database) addFragToSchemasTable(ctx *sql.Context, fragType, name, definition string, created time.Time, existingErr error) (retErr error) {
+func (db Database) addFragToSchemasTable(ctx *sql.Context, fragType, name, definition string, created time.Time, existingErr error) (err error) {
 	tbl, err := GetOrCreateDoltSchemasTable(ctx, db)
 	if err != nil {
 		return err
@@ -1022,10 +1021,12 @@ func (db Database) addFragToSchemasTable(ctx *sql.Context, fragType, name, defin
 		return existingErr
 	}
 
-	ts, err := db.TableEditSession(ctx)
+	sess := dsess.DSessFromSess(ctx.Session)
+	dbState, _, err := sess.LookupDbState(ctx, db.Name())
 	if err != nil {
 		return err
 	}
+	ts := dbState.WriteSession
 
 	ws, err := ts.Flush(ctx)
 	if err != nil {
@@ -1041,9 +1042,9 @@ func (db Database) addFragToSchemasTable(ctx *sql.Context, fragType, name, defin
 	// Insert the new row into the db
 	inserter := tbl.Inserter(ctx)
 	defer func() {
-		err := inserter.Close(ctx)
-		if retErr == nil {
-			retErr = err
+		cErr := inserter.Close(ctx)
+		if err == nil {
+			err = cErr
 		}
 	}()
 	// Encode createdAt time to JSON
@@ -1083,16 +1084,6 @@ func (db Database) dropFragFromSchemasTable(ctx *sql.Context, fragType, name str
 	return deleter.Close(ctx)
 }
 
-// TableEditSession returns the TableEditSession for this database from the given context.
-func (db Database) TableEditSession(ctx *sql.Context) (writer.WriteSession, error) {
-	sess := dsess.DSessFromSess(ctx.Session)
-	dbState, _, err := sess.LookupDbState(ctx, db.Name())
-	if err != nil {
-		return nil, err
-	}
-	return dbState.WriteSession, nil
-}
-
 // GetAllTemporaryTables returns all temporary tables
 func (db Database) GetAllTemporaryTables(ctx *sql.Context) ([]sql.Table, error) {
 	sess := dsess.DSessFromSess(ctx.Session)
