@@ -17,10 +17,13 @@ package sqle
 import (
 	"fmt"
 	"io"
+	"strings"
 
+	"github.com/dolthub/dolt/go/libraries/doltcore/diff"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/rowconv"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dtables"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
 	"github.com/dolthub/dolt/go/store/types"
@@ -41,9 +44,10 @@ type DiffTableFunction struct {
 	database       sql.Database
 	sqlSch         sql.Schema
 	joiner         *rowconv.Joiner
-	fromSch        schema.Schema
-	toSch          schema.Schema
-	diffTableSch   schema.Schema
+
+	tableDelta diff.TableDelta
+	fromDate   *types.Timestamp
+	toDate     *types.Timestamp
 }
 
 // NewInstance implements the TableFunction interface
@@ -104,7 +108,7 @@ func (dtf *DiffTableFunction) WithExpressions(expression ...sql.Expression) (sql
 		return nil, err
 	}
 
-	err = dtf.generateSchema(tableName, fromCommitVal, toCommitVal)
+	err = dtf.generateSchema(dtf.ctx, tableName, fromCommitVal, toCommitVal)
 	if err != nil {
 		return nil, err
 	}
@@ -119,77 +123,69 @@ func (dtf *DiffTableFunction) Children() []sql.Node {
 
 // RowIter implements the sql.Node interface
 func (dtf *DiffTableFunction) RowIter(ctx *sql.Context, _ sql.Row) (sql.RowIter, error) {
+	// Everything we need to start iterating was cached when we previously determined the schema of the result
 	// TODO: When we add support for joining on table functions, we'll need to evaluate this against the
 	//       specified row. That row is what has the left_table context in a join query.
 	//       This will expand the test cases we need to cover significantly.
-	tableName, fromCommitVal, toCommitVal, err := dtf.evaluateArguments()
+	_, fromCommit, toCommit, err := dtf.evaluateArguments()
 	if err != nil {
 		return nil, err
 	}
 
+	fromHash, fromOk := fromCommit.(string)
+	toHash, toOk := toCommit.(string)
+	if !fromOk || !toOk {
+		return nil, fmt.Errorf("expected strings for from and to revisions, got: %v, %v", fromHash, toHash)
+	}
+
 	sqledb, ok := dtf.database.(Database)
 	if !ok {
 		panic("unable to get dolt database")
 	}
 	ddb := sqledb.GetDoltDB()
 
-	toRoot, toHash, toDate, err := dtf.loadDetailsForRef(ctx, toCommitVal, ddb)
-	if err != nil {
-		return nil, err
-	}
-
-	toTable, _, _, err := toRoot.GetTableInsensitive(ctx, tableName)
-	if err != nil {
-		return nil, err
-	}
-
-	fromRoot, fromHash, fromDate, err := dtf.loadDetailsForRef(ctx, fromCommitVal, ddb)
-	if err != nil {
-		return nil, err
-	}
-
-	fromTable, _, _, err := fromRoot.GetTableInsensitive(ctx, tableName)
-	if err != nil {
-		return nil, err
-	}
-
-	dp := dtables.NewDiffPartition(toTable, fromTable, toHash, fromHash, toDate, fromDate, &dtf.toSch, &dtf.fromSch)
+	dp := dtables.NewDiffPartition(dtf.tableDelta.ToTable, dtf.tableDelta.FromTable, toHash, fromHash, dtf.toDate, dtf.fromDate, dtf.tableDelta.ToSch, dtf.tableDelta.FromSch)
 
 	return NewDiffTableFunctionRowIterForSinglePartition(*dp, ddb, dtf.joiner), nil
 }
 
+// findMatchingDelta returns the best matching table delta for the table name given, taking renames into account
+func findMatchingDelta(deltas []diff.TableDelta, tableName string) diff.TableDelta {
+	tableName = strings.ToLower(tableName)
+	for _, d := range deltas {
+		if strings.ToLower(d.ToName) == tableName {
+			return d
+		}
+	}
+
+	for _, d := range deltas {
+		if strings.ToLower(d.FromName) == tableName {
+			return d
+		}
+	}
+
+	// no delta means no diff, or the table doesn't exist
+	return diff.TableDelta{}
+}
+
 // loadDetailsForRef loads the root, hash, and timestamp for the specified ref value
-func (dtf *DiffTableFunction) loadDetailsForRef(ctx *sql.Context, ref interface{}, ddb *doltdb.DoltDB) (*doltdb.RootValue, string, *types.Timestamp, error) {
+func loadDetailsForRef(
+	ctx *sql.Context,
+	ref interface{},
+	ddb Database,
+) (*doltdb.RootValue, string, *types.Timestamp, error) {
 	hashStr, ok := ref.(string)
 	if !ok {
 		return nil, "", nil, fmt.Errorf("received '%v' when expecting commit hash string", ref)
 	}
 
-	var root *doltdb.RootValue
-	var commitTime *types.Timestamp
-	cs, err := doltdb.NewCommitSpec(hashStr)
+	sess := dsess.DSessFromSess(ctx.Session)
+
+	root, commitTime, err := sess.ResolveRootForRef(ctx, ddb.Name(), hashStr)
 	if err != nil {
 		return nil, "", nil, err
 	}
 
-	cm, err := ddb.Resolve(ctx, cs, nil)
-	if err != nil {
-		return nil, "", nil, err
-	}
-
-	root, err = cm.GetRootValue(ctx)
-	if err != nil {
-		return nil, "", nil, err
-	}
-
-	meta, err := cm.GetCommitMeta(ctx)
-	if err != nil {
-		return nil, "", nil, err
-	}
-
-	t := meta.Time()
-	commitTime = (*types.Timestamp)(&t)
-
 	return root, hashStr, commitTime, nil
 }
 
@@ -254,7 +250,7 @@ func (dtf *DiffTableFunction) evaluateArguments() (string, interface{}, interfac
 	return tableName, fromCommitVal, toCommitVal, nil
 }
 
-func (dtf *DiffTableFunction) generateSchema(tableName string, fromCommitVal, toCommitVal interface{}) error {
+func (dtf *DiffTableFunction) generateSchema(ctx *sql.Context, tableName string, fromCommitVal, toCommitVal interface{}) error {
 	if !dtf.Resolved() {
 		return nil
 	}
@@ -264,46 +260,32 @@ func (dtf *DiffTableFunction) generateSchema(tableName string, fromCommitVal, to
 		panic(fmt.Sprintf("unexpected database type: %T", dtf.database))
 	}
 
-	fromRoot, err := sqledb.rootAsOf(dtf.ctx, fromCommitVal)
+	delta, err := dtf.cacheTableDelta(ctx, tableName, fromCommitVal, toCommitVal, sqledb)
 	if err != nil {
 		return err
 	}
 
-	fromTable, _, ok, err := fromRoot.GetTableInsensitive(dtf.ctx, tableName)
-	if err != nil {
-		return err
-	}
-	if !ok {
+	fromTable, fromTableExists := delta.FromTable, delta.FromTable != nil
+	toTable, toTableExists := delta.ToTable, delta.ToTable != nil
+
+	if !toTableExists && !fromTableExists {
 		return sql.ErrTableNotFound.New(tableName)
 	}
 
-	toRoot, err := sqledb.rootAsOf(dtf.ctx, toCommitVal)
-	if err != nil {
-		return err
+	var toSchema, fromSchema schema.Schema
+	var format *types.NomsBinFormat
+
+	if fromTableExists {
+		fromSchema = delta.FromSch
+		format = fromTable.Format()
 	}
 
-	toTable, _, ok, err := toRoot.GetTableInsensitive(dtf.ctx, tableName)
-	if err != nil {
-		return err
-	}
-	if !ok {
-		return sql.ErrTableNotFound.New(tableName)
+	if toTableExists {
+		toSchema = delta.ToSch
+		format = toTable.Format()
 	}
 
-	fromSchema, err := fromTable.GetSchema(dtf.ctx)
-	if err != nil {
-		return err
-	}
-
-	toSchema, err := toTable.GetSchema(dtf.ctx)
-	if err != nil {
-		return err
-	}
-
-	dtf.fromSch = fromSchema
-	dtf.toSch = toSchema
-
-	diffTableSch, j, err := dtables.GetDiffTableSchemaAndJoiner(toTable.Format(), fromSchema, toSchema)
+	diffTableSch, j, err := dtables.GetDiffTableSchemaAndJoiner(format, fromSchema, toSchema)
 	if err != nil {
 		return err
 	}
@@ -324,6 +306,74 @@ func (dtf *DiffTableFunction) generateSchema(tableName string, fromCommitVal, to
 	return nil
 }
 
+// cacheTableDelta caches and returns an appropriate table delta for the table name given, taking renames into
+// consideration. Returns a sql.ErrTableNotFound if the given table name cannot be found in either revision.
+func (dtf *DiffTableFunction) cacheTableDelta(ctx *sql.Context, tableName string, fromCommitVal interface{}, toCommitVal interface{}, db Database) (diff.TableDelta, error) {
+	fromRoot, _, fromDate, err := loadDetailsForRef(ctx, fromCommitVal, db)
+	if err != nil {
+		return diff.TableDelta{}, err
+	}
+
+	toRoot, _, toDate, err := loadDetailsForRef(ctx, toCommitVal, db)
+	if err != nil {
+		return diff.TableDelta{}, err
+	}
+
+	fromTable, _, fromTableExists, err := fromRoot.GetTableInsensitive(ctx, tableName)
+	if err != nil {
+		return diff.TableDelta{}, err
+	}
+	toTable, _, toTableExists, err := toRoot.GetTableInsensitive(ctx, tableName)
+	if err != nil {
+		return diff.TableDelta{}, err
+	}
+
+	if !fromTableExists && !toTableExists {
+		return diff.TableDelta{}, sql.ErrTableNotFound.New(tableName)
+	}
+
+	// TODO: it would be nice to limit this to just the table under consideration, not all tables with a diff
+	deltas, err := diff.GetTableDeltas(ctx, fromRoot, toRoot)
+	if err != nil {
+		return diff.TableDelta{}, err
+	}
+
+	dtf.fromDate = fromDate
+	dtf.toDate = toDate
+
+	delta := findMatchingDelta(deltas, tableName)
+
+	// We only get a delta if there's a diff. When there isn't one, construct a delta here with table and schema info
+	if delta.FromTable == nil && delta.ToTable == nil {
+		delta.FromName = tableName
+		delta.ToName = tableName
+		delta.FromTable = fromTable
+		delta.ToTable = toTable
+
+		if fromTable != nil {
+			sch, err := fromTable.GetSchema(ctx)
+			if err != nil {
+				return diff.TableDelta{}, err
+			}
+			delta.FromSch = sch
+		}
+
+		if toTable != nil {
+			sch, err := toTable.GetSchema(ctx)
+			if err != nil {
+				return diff.TableDelta{}, err
+			}
+			delta.ToSch = sch
+		}
+
+		// TODO: There are other fields we could set here that we don't
+	}
+
+	dtf.tableDelta = delta
+
+	return delta, nil
+}
+
 // Schema implements the sql.Node interface
 func (dtf *DiffTableFunction) Schema() sql.Schema {
 	if !dtf.Resolved() {
