@@ -16,8 +16,12 @@ package dsess
 
 import (
 	"github.com/dolthub/go-mysql-server/sql"
+	"gopkg.in/src-d/go-errors.v1"
 )
 
+// ErrRevisionDbNotFound is thrown when a RevisionDatabaseProvider cannot find a specified revision database.
+var ErrRevisionDbNotFound = errors.NewKind("revision database not found: '%s'")
+
 // RevisionDatabaseProvider provides revision databases.
 // In Dolt, commits and branches can be accessed as discrete databases
 // using a Dolt-specific syntax: `my_database/my_branch`. Revision databases
@@ -26,6 +30,8 @@ import (
 type RevisionDatabaseProvider interface {
 	// RevisionDbState provides the InitialDbState for a revision database.
 	RevisionDbState(ctx *sql.Context, revDB string) (InitialDbState, error)
+	// DropRevisionDb removes the specified revision database from the databases this provider is tracking.
+	DropRevisionDb(ctx *sql.Context, revDB string) error
 }
 
 func EmptyDatabaseProvider() RevisionDatabaseProvider {
@@ -34,6 +40,10 @@ func EmptyDatabaseProvider() RevisionDatabaseProvider {
 
 type emptyRevisionDatabaseProvider struct{}
 
+func (e emptyRevisionDatabaseProvider) DropRevisionDb(ctx *sql.Context, revDB string) error {
+	return nil
+}
+
 func (e emptyRevisionDatabaseProvider) RevisionDbState(_ *sql.Context, revDB string) (InitialDbState, error) {
 	return InitialDbState{}, sql.ErrDatabaseNotFound.New(revDB)
 }
