@@ -671,4 +671,22 @@ var BrokenDDLScripts = []queries.ScriptTest{
 			},
 		},
 	},
+	{
+		// This actually "works", incorrectly
+		// We need to disambiguate the column names when they collide with generated ones
+		// https://github.com/dolthub/dolt/issues/3395
+		Name: "table with commit column should maintain its data in diff",
+		SetUpScript: []string{
+			"CREATE TABLE t (pk int PRIMARY KEY, commit text);",
+			"set @Commit1 = dolt_commit('-am', 'creating table t');",
+			"INSERT INTO t VALUES (1, 'hi');",
+			"set @Commit2 = dolt_commit('-am', 'insert data');",
+		},
+		Assertions: []queries.ScriptTestAssertion{
+			{
+				Query:    "SELECT to_pk, to_commit, from_pk, from_commit, diff_type from dolt_diff('t', @Commit1, @Commit2);",
+				Expected: []sql.Row{{1, "hi", nil, nil, "added"}},
+			},
+		},
+	},
 }
