@@ -80,30 +80,27 @@ func TestSingleQuery(t *testing.T) {
 
 // Convenience test for debugging a single query. Unskip and set to the desired query.
 func TestSingleScript(t *testing.T) {
-	t.Skip()
+	// t.Skip()
 
 	var scripts = []queries.ScriptTest{
 		{
-			Name: "primary key table: non-pk column type changes",
+			Name: "new table",
 			SetUpScript: []string{
-				"create table t (pk int primary key, c1 int, c2 text);",
-				"insert into t values (1, 2, '3'), (4, 5, '6');",
-				"set @Commit1 = DOLT_COMMIT('-am', 'creating table t');",
-				"alter table t modify column c2 int;",
-				"set @Commit2 = DOLT_COMMIT('-am', 'changed type of c2');",
+				"create table t1 (a int primary key, b int)",
+				"insert into t1 values (1,2)",
 			},
 			Assertions: []queries.ScriptTestAssertion{
 				{
-					Query:    "select count(*) from dolt_history_t;",
-					Expected: []sql.Row{{4}},
+					Query:    "select to_a, to_b, from_commit, to_commit, diff_type from dolt_diff('t1', 'HEAD', 'WORKING')",
+					Expected: []sql.Row{{1, 2, "HEAD", "WORKING", "added"}},
 				},
 				{
-					Query:    "select pk, c2 from dolt_history_t where commit_hash=@Commit1 order by pk;",
-					Expected: []sql.Row{{1, nil}, {4, nil}},
+					Query:       "select to_a, from_b, from_commit, to_commit, diff_type from dolt_diff('t1', 'HEAD', 'WORKING')",
+					ExpectedErr: sql.ErrColumnNotFound,
 				},
 				{
-					Query:    "select pk, c2 from dolt_history_t where commit_hash=@Commit2 order by pk;",
-					Expected: []sql.Row{{1, 3}, {4, 6}},
+					Query:    "select from_a, from_b, from_commit, to_commit, diff_type from dolt_diff('t1', 'WORKING', 'HEAD')",
+					Expected: []sql.Row{{1, 2, "WORKING", "HEAD", "removed"}},
 				},
 			},
 		},
@@ -695,6 +692,16 @@ func TestDoltMerge(t *testing.T) {
 	}
 }
 
+// tests new format behavior for keyless merges that create CVs and conflicts
+func TestKeylessDoltMergeCVsAndConflicts(t *testing.T) {
+	if !types.IsFormat_DOLT_1(types.Format_Default) {
+		t.Skip()
+	}
+	for _, script := range KeylessMergeCVsAndConflictsScripts {
+		enginetest.TestScript(t, newDoltHarness(t), script)
+	}
+}
+
 // eventually this will be part of TestDoltMerge
 func TestDoltMergeArtifacts(t *testing.T) {
 	if !types.IsFormat_DOLT_1(types.Format_Default) {
