@@ -25,6 +25,7 @@ import (
 	"time"
 
 	"github.com/dolthub/go-mysql-server/sql"
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
@@ -1441,3 +1442,69 @@ func convertSqlRowToInt64(sqlRows []sql.Row) []sql.Row {
 	}
 	return newSqlRows
 }
+
+func TestSplitNullsFromRange(t *testing.T) {
+	t.Run("EmptyRange", func(t *testing.T) {
+		r, err := index.SplitNullsFromRange(sql.Range{})
+		assert.NoError(t, err)
+		assert.NotNil(t, r)
+		assert.Len(t, r, 1)
+		assert.Len(t, r[0], 0)
+	})
+
+	t.Run("ThreeColumnNoNullsRange", func(t *testing.T) {
+		r := sql.Range{sql.LessThanRangeColumnExpr(10, sql.Int8), sql.GreaterThanRangeColumnExpr(16, sql.Int8), sql.NotNullRangeColumnExpr(sql.Int8)}
+		rs, err := index.SplitNullsFromRange(r)
+		assert.NoError(t, err)
+		assert.NotNil(t, rs)
+		assert.Len(t, rs, 1)
+		assert.Len(t, rs[0], 3)
+		assert.Equal(t, r, rs[0])
+	})
+
+	t.Run("LastColumnOnlyNull", func(t *testing.T) {
+		r := sql.Range{sql.LessThanRangeColumnExpr(10, sql.Int8), sql.GreaterThanRangeColumnExpr(16, sql.Int8), sql.NullRangeColumnExpr(sql.Int8)}
+		rs, err := index.SplitNullsFromRange(r)
+		assert.NoError(t, err)
+		assert.NotNil(t, rs)
+		assert.Len(t, rs, 1)
+		assert.Len(t, rs[0], 3)
+		assert.Equal(t, r, rs[0])
+	})
+
+	t.Run("LastColumnAll", func(t *testing.T) {
+		r := sql.Range{sql.LessThanRangeColumnExpr(10, sql.Int8), sql.GreaterThanRangeColumnExpr(16, sql.Int8), sql.AllRangeColumnExpr(sql.Int8)}
+		rs, err := index.SplitNullsFromRange(r)
+		assert.NoError(t, err)
+		assert.NotNil(t, rs)
+		assert.Len(t, rs, 2)
+		assert.Len(t, rs[0], 3)
+		assert.Len(t, rs[1], 3)
+		assert.Equal(t, r[:2], rs[0][:2])
+		assert.Equal(t, r[:2], rs[1][:2])
+		assert.Equal(t, sql.NullRangeColumnExpr(sql.Int8), rs[0][2])
+		assert.Equal(t, sql.NotNullRangeColumnExpr(sql.Int8), rs[1][2])
+	})
+
+	t.Run("FirstColumnAll", func(t *testing.T) {
+		r := sql.Range{sql.AllRangeColumnExpr(sql.Int8), sql.LessThanRangeColumnExpr(10, sql.Int8), sql.GreaterThanRangeColumnExpr(16, sql.Int8)}
+		rs, err := index.SplitNullsFromRange(r)
+		assert.NoError(t, err)
+		assert.NotNil(t, rs)
+		assert.Len(t, rs, 2)
+		assert.Len(t, rs[0], 3)
+		assert.Len(t, rs[1], 3)
+		assert.Equal(t, r[1:], rs[0][1:])
+		assert.Equal(t, r[1:], rs[1][1:])
+		assert.Equal(t, sql.NullRangeColumnExpr(sql.Int8), rs[0][0])
+		assert.Equal(t, sql.NotNullRangeColumnExpr(sql.Int8), rs[1][0])
+	})
+
+	t.Run("AllColumnAll", func(t *testing.T) {
+		r := sql.Range{sql.AllRangeColumnExpr(sql.Int8), sql.AllRangeColumnExpr(sql.Int8), sql.AllRangeColumnExpr(sql.Int8)}
+		rs, err := index.SplitNullsFromRange(r)
+		assert.NoError(t, err)
+		assert.NotNil(t, rs)
+		assert.Len(t, rs, 8)
+	})
+}
