@@ -55,20 +55,9 @@ type indexLookupRowIterAdapter struct {
 }
 
 // NewIndexLookupRowIterAdapter returns a new indexLookupRowIterAdapter.
-func NewIndexLookupRowIterAdapter(ctx *sql.Context, idx DoltIndex, tableData durable.Index, keyIter nomsKeyIter) (*indexLookupRowIterAdapter, error) {
-	lookupTags := make(map[uint64]int)
-	for i, tag := range idx.Schema().GetPKCols().Tags {
-		lookupTags[tag] = i
-	}
+func NewIndexLookupRowIterAdapter(ctx *sql.Context, idx DoltIndex, durableState *durableIndexState, keyIter nomsKeyIter) (*indexLookupRowIterAdapter, error) {
+	rows := durable.NomsMapFromIndex(durableState.Primary)
 
-	// handle keyless case, where no columns are pk's and rowIdTag is the only lookup tag
-	if len(lookupTags) == 0 {
-		lookupTags[schema.KeylessRowIdTag] = 0
-	}
-
-	rows := durable.NomsMapFromIndex(tableData)
-
-	conv := NewKVToSqlRowConverterForCols(idx.Format(), idx.Schema())
 	resBuf := resultBufferPool.Get().(*async.RingBuffer)
 	epoch := resBuf.Reset()
 
@@ -78,8 +67,8 @@ func NewIndexLookupRowIterAdapter(ctx *sql.Context, idx DoltIndex, tableData dur
 		idx:        idx,
 		keyIter:    keyIter,
 		tableRows:  rows,
-		conv:       conv,
-		lookupTags: lookupTags,
+		conv:       idx.sqlRowConverter(durableState),
+		lookupTags: idx.lookupTags(durableState),
 		cancelF:    cancelF,
 		resultBuf:  resBuf,
 	}
