@@ -113,6 +113,11 @@ func TestRoundTripProllyFields(t *testing.T) {
 			typ:   val.Type{Enc: val.StringEnc},
 			value: "lorem ipsum",
 		},
+		{
+			name:  "string",
+			typ:   val.Type{Enc: val.StringAddrEnc},
+			value: "lorem ipsum",
+		},
 		{
 			name:  "bytes",
 			typ:   val.Type{Enc: val.ByteStringEnc},
@@ -145,7 +150,7 @@ func TestRoundTripProllyFields(t *testing.T) {
 		},
 		{
 			name:  "json",
-			typ:   val.Type{Enc: val.JSONEnc},
+			typ:   val.Type{Enc: val.JSONAddrEnc},
 			value: mustParseJson(t, `{"a": 1, "b": false}`),
 		},
 		{
