@@ -16,7 +16,6 @@ package sqlfmt
 
 import (
 	"bytes"
-	"context"
 	"fmt"
 	"strings"
 
@@ -92,8 +91,101 @@ func RowAsInsertStmt(r row.Row, tableName string, tableSch schema.Schema) (strin
 	return b.String(), nil
 }
 
+func RowAsDeleteStmt(r row.Row, tableName string, tableSch schema.Schema) (string, error) {
+	var b strings.Builder
+	b.WriteString("DELETE FROM ")
+	b.WriteString(QuoteIdentifier(tableName))
+
+	b.WriteString(" WHERE (")
+	seenOne := false
+	isKeyless := tableSch.GetPKCols().Size() == 0
+	_, err := r.IterSchema(tableSch, func(tag uint64, val types.Value) (stop bool, err error) {
+		col, _ := tableSch.GetAllCols().GetByTag(tag)
+		if col.IsPartOfPK || isKeyless {
+			if seenOne {
+				b.WriteString(" AND ")
+			}
+			sqlString, err := valueAsSqlString(col.TypeInfo, val)
+			if err != nil {
+				return true, err
+			}
+			b.WriteString(QuoteIdentifier(col.Name))
+			b.WriteRune('=')
+			b.WriteString(sqlString)
+			seenOne = true
+		}
+		return false, nil
+	})
+
+	if err != nil {
+		return "", err
+	}
+
+	b.WriteString(");")
+	return b.String(), nil
+}
+
+func RowAsUpdateStmt(r row.Row, tableName string, tableSch schema.Schema, colsToUpdate *set.StrSet) (string, error) {
+	var b strings.Builder
+	b.WriteString("UPDATE ")
+	b.WriteString(QuoteIdentifier(tableName))
+	b.WriteString(" ")
+
+	b.WriteString("SET ")
+	seenOne := false
+	_, err := r.IterSchema(tableSch, func(tag uint64, val types.Value) (stop bool, err error) {
+		col, _ := tableSch.GetAllCols().GetByTag(tag)
+		exists := colsToUpdate.Contains(col.Name)
+		if !col.IsPartOfPK && exists {
+			if seenOne {
+				b.WriteRune(',')
+			}
+			sqlString, err := valueAsSqlString(col.TypeInfo, val)
+			if err != nil {
+				return true, err
+			}
+			b.WriteString(QuoteIdentifier(col.Name))
+			b.WriteRune('=')
+			b.WriteString(sqlString)
+			seenOne = true
+		}
+		return false, nil
+	})
+
+	if err != nil {
+		return "", err
+	}
+
+	b.WriteString(" WHERE (")
+	seenOne = false
+	_, err = r.IterSchema(tableSch, func(tag uint64, val types.Value) (stop bool, err error) {
+		col, _ := tableSch.GetAllCols().GetByTag(tag)
+		if col.IsPartOfPK {
+			if seenOne {
+				b.WriteString(" AND ")
+			}
+			sqlString, err := valueAsSqlString(col.TypeInfo, val)
+			if err != nil {
+				return true, err
+			}
+			b.WriteString(QuoteIdentifier(col.Name))
+			b.WriteRune('=')
+			b.WriteString(sqlString)
+			seenOne = true
+		}
+		return false, nil
+	})
+
+	if err != nil {
+		return "", err
+	}
+
+	b.WriteString(");")
+	return b.String(), nil
+}
+
 // RowAsTupleString converts a row into it's tuple string representation for SQL insert statements.
-func RowAsTupleString(ctx context.Context, r row.Row, tableSch schema.Schema) (string, error) {
+func RowAsTupleString(r row.Row, tableSch schema.Schema) (string, error) {
 	var b strings.Builder
 
 	b.WriteString("(")
@@ -210,7 +302,7 @@ func SqlRowAsCreateFragStmt(r sql.Row) (string, error) {
 	return b.String(), nil
 }
 
-func SqlRowAsInsertStmt(ctx context.Context, r sql.Row, tableName string, tableSch schema.Schema) (string, error) {
+func SqlRowAsInsertStmt(r sql.Row, tableName string, tableSch schema.Schema) (string, error) {
 	var b strings.Builder
 
 	// Write insert prefix
@@ -221,7 +313,7 @@ func SqlRowAsInsertStmt(ctx context.Context, r sql.Row, tableName string, tableS
 	b.WriteString(prefix)
 
 	// Write single insert
-	str, err := SqlRowAsTupleString(ctx, r, tableSch)
+	str, err := SqlRowAsTupleString(r, tableSch)
 	if err != nil {
 		return "", err
 	}
@@ -232,7 +324,7 @@ func SqlRowAsInsertStmt(ctx context.Context, r sql.Row, tableName string, tableS
 }
 
 // SqlRowAsTupleString converts a sql row into it's tuple string representation for SQL insert statements.
-func SqlRowAsTupleString(ctx context.Context, r sql.Row, tableSch schema.Schema) (string, error) {
+func SqlRowAsTupleString(r sql.Row, tableSch schema.Schema) (string, error) {
 	var b strings.Builder
 	var err error
 
@@ -245,7 +337,7 @@ func SqlRowAsTupleString(ctx context.Context, r sql.Row, tableSch schema.Schema)
 		col := tableSch.GetAllCols().GetAtIndex(i)
 		str := "NULL"
 		if val != nil {
-			str, err = interfaceValueAsSqlString(ctx, col.TypeInfo, val)
+			str, err = interfaceValueAsSqlString(col.TypeInfo, val)
 			if err != nil {
 				return "", err
 			}
@@ -259,21 +351,22 @@ func SqlRowAsTupleString(ctx context.Context, r sql.Row, tableSch schema.Schema)
 	return b.String(), nil
 }
 
-func RowAsDeleteStmt(r row.Row, tableName string, tableSch schema.Schema) (string, error) {
+func SqlRowAsDeleteStmt(r sql.Row, tableName string, tableSch schema.Schema) (string, error) {
 	var b strings.Builder
 	b.WriteString("DELETE FROM ")
 	b.WriteString(QuoteIdentifier(tableName))
 
-	b.WriteString(" WHERE (")
+	b.WriteString(" WHERE ")
 	seenOne := false
-	isKeyless := tableSch.GetPKCols().Size() == 0
-	_, err := r.IterSchema(tableSch, func(tag uint64, val types.Value) (stop bool, err error) {
-		col, _ := tableSch.GetAllCols().GetByTag(tag)
+	i := 0
+	isKeyless := schema.IsKeyless(tableSch)
+
+	err := tableSch.GetAllCols().Iter(func(_ uint64, col schema.Column) (stop bool, err error) {
 		if col.IsPartOfPK || isKeyless {
 			if seenOne {
 				b.WriteString(" AND ")
 			}
-			sqlString, err := valueAsSqlString(col.TypeInfo, val)
+			sqlString, err := interfaceValueAsSqlString(col.TypeInfo, r[i])
 			if err != nil {
 				return true, err
 			}
@@ -282,6 +375,7 @@ func RowAsDeleteStmt(r row.Row, tableName string, tableSch schema.Schema) (strin
 			b.WriteString(sqlString)
 			seenOne = true
 		}
+		i++
 		return false, nil
 	})
 
@@ -289,34 +383,36 @@ func RowAsDeleteStmt(r row.Row, tableName string, tableSch schema.Schema) (strin
 		return "", err
 	}
 
-	b.WriteString(");")
+	b.WriteString(";")
 	return b.String(), nil
 }
 
-func RowAsUpdateStmt(r row.Row, tableName string, tableSch schema.Schema, colDiffs *set.StrSet) (string, error) {
+func SqlRowAsUpdateStmt(r sql.Row, tableName string, tableSch schema.Schema, colsToUpdate *set.StrSet) (string, error) {
 	var b strings.Builder
 	b.WriteString("UPDATE ")
 	b.WriteString(QuoteIdentifier(tableName))
 	b.WriteString(" ")
 
 	b.WriteString("SET ")
+
+	i := 0
 	seenOne := false
-	_, err := r.IterSchema(tableSch, func(tag uint64, val types.Value) (stop bool, err error) {
-		col, _ := tableSch.GetAllCols().GetByTag(tag)
-		exists := colDiffs.Contains(col.Name)
-		if !col.IsPartOfPK && exists {
+	err := tableSch.GetAllCols().Iter(func(_ uint64, col schema.Column) (stop bool, err error) {
+		if colsToUpdate.Contains(col.Name) {
 			if seenOne {
 				b.WriteRune(',')
 			}
-			sqlString, err := valueAsSqlString(col.TypeInfo, val)
+			seenOne = true
+
+			sqlString, err := interfaceValueAsSqlString(col.TypeInfo, r[i])
 			if err != nil {
 				return true, err
 			}
 			b.WriteString(QuoteIdentifier(col.Name))
 			b.WriteRune('=')
 			b.WriteString(sqlString)
-			seenOne = true
 		}
+		i++
 		return false, nil
 	})
 
@@ -324,23 +420,26 @@ func RowAsUpdateStmt(r row.Row, tableName string, tableSch schema.Schema, colDif
 		return "", err
 	}
 
-	b.WriteString(" WHERE (")
+	b.WriteString(" WHERE ")
+
+	i = 0
 	seenOne = false
-	_, err = r.IterSchema(tableSch, func(tag uint64, val types.Value) (stop bool, err error) {
-		col, _ := tableSch.GetAllCols().GetByTag(tag)
+	err = tableSch.GetAllCols().Iter(func(_ uint64, col schema.Column) (stop bool, err error) {
 		if col.IsPartOfPK {
 			if seenOne {
 				b.WriteString(" AND ")
 			}
-			sqlString, err := valueAsSqlString(col.TypeInfo, val)
+			seenOne = true
+
+			sqlString, err := interfaceValueAsSqlString(col.TypeInfo, r[i])
 			if err != nil {
 				return true, err
 			}
 			b.WriteString(QuoteIdentifier(col.Name))
 			b.WriteRune('=')
 			b.WriteString(sqlString)
-			seenOne = true
 		}
+		i++
 		return false, nil
 	})
 
@@ -348,7 +447,7 @@ func RowAsUpdateStmt(r row.Row, tableName string, tableSch schema.Schema, colDif
 		return "", err
 	}
 
-	b.WriteString(");")
+	b.WriteString(";")
 	return b.String(), nil
 }
 
@@ -385,19 +484,18 @@ func valueAsSqlString(ti typeinfo.TypeInfo, value types.Value) (string, error) {
 	}
 }
 
-func interfaceValueAsSqlString(ctx context.Context, ti typeinfo.TypeInfo, value interface{}) (string, error) {
-	str, err := sqlutil.SqlColToStr(ctx, ti.ToSqlType(), value)
+func interfaceValueAsSqlString(ti typeinfo.TypeInfo, value interface{}) (string, error) {
+	str, err := sqlutil.SqlColToStr(ti.ToSqlType(), value)
 	if err != nil {
 		return "", err
 	}
 
 	switch ti.GetTypeIdentifier() {
 	case typeinfo.BoolTypeIdentifier:
-		// todo: unclear if we want this to output with "TRUE/FALSE" or 1/0
 		if value.(bool) {
-			return "TRUE", nil
+			return "1", nil
 		}
-		return "FALSE", nil
+		return "0", nil
 	case typeinfo.UuidTypeIdentifier, typeinfo.TimeTypeIdentifier, typeinfo.YearTypeIdentifier:
 		return singleQuote + str + singleQuote, nil
 	case typeinfo.DatetimeTypeIdentifier:
