@@ -17,9 +17,12 @@ package tree
 import (
 	"bytes"
 	"context"
+	"encoding/json"
 	"errors"
 	"io"
 
+	"github.com/dolthub/go-mysql-server/sql"
+
 	"github.com/dolthub/dolt/go/store/hash"
 	"github.com/dolthub/dolt/go/store/prolly/message"
 	"github.com/dolthub/dolt/go/store/val"
@@ -152,6 +155,8 @@ func _newLeaf(ctx context.Context, ns NodeStore, s message.Serializer, buf []byt
 	}, nil
 }
 
+const bytePeekLength = 128
+
 type ByteArray struct {
 	ImmutableTree
 }
@@ -161,27 +166,72 @@ func NewByteArray(addr hash.Hash, ns NodeStore) *ByteArray {
 }
 
 func (b *ByteArray) ToBytes(ctx context.Context) ([]byte, error) {
-	if b.buf == nil {
-		err := b.load(ctx)
-		if err != nil {
-			return nil, err
-		}
-	}
-	return b.buf[:], nil
+	return b.bytes(ctx)
 }
 
 func (b *ByteArray) ToString(ctx context.Context) (string, error) {
-	if b.buf == nil {
-		err := b.load(ctx)
-		if err != nil {
-			return "", err
-		}
+	buf, err := b.bytes(ctx)
+	if err != nil {
+		return "", err
 	}
-	toShow := 128
-	if len(b.buf) < toShow {
-		toShow = len(b.buf)
+	toShow := bytePeekLength
+	if len(buf) < toShow {
+		toShow = len(buf)
 	}
-	return string(b.buf[:toShow]), nil
+	return string(buf[:toShow]), nil
+}
+
+type JSONDoc struct {
+	ImmutableTree
+}
+
+func NewJSONDoc(addr hash.Hash, ns NodeStore) *JSONDoc {
+	return &JSONDoc{ImmutableTree{Addr: addr, ns: ns}}
+}
+
+func (b *JSONDoc) ToJSONDocument(ctx context.Context) (sql.JSONDocument, error) {
+	buf, err := b.bytes(ctx)
+	if err != nil {
+		return sql.JSONDocument{}, err
+	}
+	var doc sql.JSONDocument
+	err = json.Unmarshal(buf, &doc.Val)
+	if err != nil {
+		return sql.JSONDocument{}, err
+	}
+	return doc, err
+}
+
+func (b *JSONDoc) ToString(ctx context.Context) (string, error) {
+	buf, err := b.bytes(ctx)
+	if err != nil {
+		return "", err
+	}
+	toShow := bytePeekLength
+	if len(buf) < toShow {
+		toShow = len(buf)
+	}
+	return string(buf[:toShow]), nil
+}
+
+type TextStorage struct {
+	ImmutableTree
+}
+
+func NewTextStorage(addr hash.Hash, ns NodeStore) *TextStorage {
+	return &TextStorage{ImmutableTree{Addr: addr, ns: ns}}
+}
+
+func (b *TextStorage) ToBytes(ctx context.Context) ([]byte, error) {
+	return b.bytes(ctx)
+}
+
+func (b *TextStorage) ToString(ctx context.Context) (string, error) {
+	buf, err := b.bytes(ctx)
+	if err != nil {
+		return "", err
+	}
+	return string(buf), nil
 }
 
 type ImmutableTree struct {
@@ -220,6 +270,16 @@ func (t *ImmutableTree) load(ctx context.Context) error {
 	return nil
 }
 
+func (t *ImmutableTree) bytes(ctx context.Context) ([]byte, error) {
+	if t.buf == nil {
+		err := t.load(ctx)
+		if err != nil {
+			return nil, err
+		}
+	}
+	return t.buf[:], nil
+}
+
 func (t *ImmutableTree) next() (Node, error) {
 	panic("not implemented")
 }
