@@ -15,6 +15,7 @@
 package val
 
 import (
+	"bytes"
 	"math"
 
 	"github.com/dolthub/dolt/go/store/pool"
@@ -111,6 +112,15 @@ func NewTuple(pool pool.BuffPool, values ...[]byte) Tuple {
 	return tup
 }
 
+func CellWiseTupleDiff(from, to Tuple) (c uint64) {
+	for i := 0; i < from.Count(); i++ {
+		if !bytes.Equal(from.GetField(i), to.GetField(i)) {
+			c++
+		}
+	}
+	return
+}
+
 func cloneTuple(pool pool.BuffPool, tup Tuple) Tuple {
 	buf := pool.Get(uint64(len(tup)))
 	copy(buf, tup)
